@@ -27,7 +27,6 @@ from samba import gensec
 from samba.credentials import Credentials
 from samba.tests import TestCase
 from samba.ndr import ndr_pack, ndr_unpack, ndr_unpack_out
-from samba.compat import text_type
 from samba.ntstatus import (
     NT_STATUS_CONNECTION_DISCONNECTED,
     NT_STATUS_PIPE_DISCONNECTED,
@@ -36,14 +35,18 @@ from samba.ntstatus import (
 from samba import NTSTATUSError
 from samba.samba3 import param as s3param
 from samba.samba3 import libsmb_samba_internal as libsmb
+from samba.credentials import SMB_SIGNING_REQUIRED
 
 class smb_pipe_socket(object):
 
     def __init__(self, target_hostname, pipename, creds, impersonation_level, lp):
         lp3 = s3param.get_context()
         lp3.load(lp.configfile)
+        saved_signing_state = creds.get_smb_ipc_signing()
+        creds.set_smb_ipc_signing(SMB_SIGNING_REQUIRED)
         self.smbconn = libsmb.Conn(target_hostname, 'IPC$', lp3,
-                                   creds=creds, sign=True)
+                                   creds=creds, ipc=True)
+        creds.set_smb_ipc_signing(saved_signing_state)
         self.smbfid = self.smbconn.create(pipename,
                                           DesiredAccess=0x12019f,
                                           ShareAccess=0x7,
@@ -118,9 +121,9 @@ class RawDCERPCTest(TestCase):
 
     def _connect_smb(self):
         a = self.primary_address.split('\\')
-        self.assertEquals(len(a), 3)
-        self.assertEquals(a[0], "")
-        self.assertEquals(a[1], "pipe")
+        self.assertEqual(len(a), 3)
+        self.assertEqual(a[0], "")
+        self.assertEqual(a[1], "pipe")
         pipename = a[2]
         self.s = smb_pipe_socket(self.target_hostname,
                                  pipename,
@@ -318,21 +321,21 @@ class RawDCERPCTest(TestCase):
                                 pfc_flags=req.pfc_flags |
                                 samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_DID_NOT_EXECUTE,
                                 auth_length=0)
-                self.assertNotEquals(rep.u.alloc_hint, 0)
-                self.assertEquals(rep.u.context_id, 0)
-                self.assertEquals(rep.u.cancel_count, 0)
-                self.assertEquals(rep.u.flags, 0)
-                self.assertEquals(rep.u.status, alter_fault)
-                self.assertEquals(rep.u.reserved, 0)
-                self.assertEquals(len(rep.u.error_and_verifier), 0)
+                self.assertNotEqual(rep.u.alloc_hint, 0)
+                self.assertEqual(rep.u.context_id, 0)
+                self.assertEqual(rep.u.cancel_count, 0)
+                self.assertEqual(rep.u.flags, 0)
+                self.assertEqual(rep.u.status, alter_fault)
+                self.assertEqual(rep.u.reserved, 0)
+                self.assertEqual(len(rep.u.error_and_verifier), 0)
                 return None
             self.verify_pdu(rep, samba.dcerpc.dcerpc.DCERPC_PKT_ALTER_RESP, req.call_id,
                             pfc_flags=req.pfc_flags)
-            self.assertEquals(rep.u.max_xmit_frag, req.u.max_xmit_frag)
-            self.assertEquals(rep.u.max_recv_frag, req.u.max_recv_frag)
-            self.assertEquals(rep.u.assoc_group_id, assoc_group_id)
-            self.assertEquals(rep.u.secondary_address_size, 0)
-            self.assertEquals(rep.u.secondary_address, '')
+            self.assertEqual(rep.u.max_xmit_frag, req.u.max_xmit_frag)
+            self.assertEqual(rep.u.max_recv_frag, req.u.max_recv_frag)
+            self.assertEqual(rep.u.assoc_group_id, assoc_group_id)
+            self.assertEqual(rep.u.secondary_address_size, 0)
+            self.assertEqual(rep.u.secondary_address, '')
             self.assertPadding(rep.u._pad1, 2)
         else:
             req = self.generate_bind(call_id=call_id,
@@ -345,20 +348,20 @@ class RawDCERPCTest(TestCase):
             if nak_reason is not None:
                 self.verify_pdu(rep, samba.dcerpc.dcerpc.DCERPC_PKT_BIND_NAK, req.call_id,
                                 auth_length=0)
-                self.assertEquals(rep.u.reject_reason, nak_reason)
-                self.assertEquals(rep.u.num_versions, 1)
-                self.assertEquals(rep.u.versions[0].rpc_vers, req.rpc_vers)
-                self.assertEquals(rep.u.versions[0].rpc_vers_minor, req.rpc_vers_minor)
+                self.assertEqual(rep.u.reject_reason, nak_reason)
+                self.assertEqual(rep.u.num_versions, 1)
+                self.assertEqual(rep.u.versions[0].rpc_vers, req.rpc_vers)
+                self.assertEqual(rep.u.versions[0].rpc_vers_minor, req.rpc_vers_minor)
                 self.assertPadding(rep.u._pad, 3)
                 return
             self.verify_pdu(rep, samba.dcerpc.dcerpc.DCERPC_PKT_BIND_ACK, req.call_id,
                             pfc_flags=pfc_flags)
-            self.assertEquals(rep.u.max_xmit_frag, req.u.max_xmit_frag)
-            self.assertEquals(rep.u.max_recv_frag, req.u.max_recv_frag)
+            self.assertEqual(rep.u.max_xmit_frag, req.u.max_xmit_frag)
+            self.assertEqual(rep.u.max_recv_frag, req.u.max_recv_frag)
             if assoc_group_id != 0:
-                self.assertEquals(rep.u.assoc_group_id, assoc_group_id)
+                self.assertEqual(rep.u.assoc_group_id, assoc_group_id)
             else:
-                self.assertNotEquals(rep.u.assoc_group_id, 0)
+                self.assertNotEqual(rep.u.assoc_group_id, 0)
                 assoc_group_id = rep.u.assoc_group_id
             sda_str = self.secondary_address
             sda_len = len(sda_str) + 1
@@ -367,24 +370,24 @@ class RawDCERPCTest(TestCase):
                 sda_pad = 4 - mod_len
             else:
                 sda_pad = 0
-            self.assertEquals(rep.u.secondary_address_size, sda_len)
-            self.assertEquals(rep.u.secondary_address, sda_str)
+            self.assertEqual(rep.u.secondary_address_size, sda_len)
+            self.assertEqual(rep.u.secondary_address, sda_str)
             self.assertPadding(rep.u._pad1, sda_pad)
 
-        self.assertEquals(rep.u.num_results, 1)
-        self.assertEquals(rep.u.ctx_list[0].result,
+        self.assertEqual(rep.u.num_results, 1)
+        self.assertEqual(rep.u.ctx_list[0].result,
                           samba.dcerpc.dcerpc.DCERPC_BIND_ACK_RESULT_ACCEPTANCE)
-        self.assertEquals(rep.u.ctx_list[0].reason,
+        self.assertEqual(rep.u.ctx_list[0].reason,
                           samba.dcerpc.dcerpc.DCERPC_BIND_ACK_REASON_NOT_SPECIFIED)
         self.assertNDRSyntaxEquals(rep.u.ctx_list[0].syntax, ctx.transfer_syntaxes[0])
         ack = rep
         if auth_context is None:
-            self.assertEquals(rep.auth_length, 0)
-            self.assertEquals(len(rep.u.auth_info), 0)
+            self.assertEqual(rep.auth_length, 0)
+            self.assertEqual(len(rep.u.auth_info), 0)
             return ack
-        self.assertNotEquals(rep.auth_length, 0)
+        self.assertNotEqual(rep.auth_length, 0)
         self.assertGreater(len(rep.u.auth_info), samba.dcerpc.dcerpc.DCERPC_AUTH_TRAILER_LENGTH)
-        self.assertEquals(rep.auth_length, len(rep.u.auth_info) - samba.dcerpc.dcerpc.DCERPC_AUTH_TRAILER_LENGTH)
+        self.assertEqual(rep.auth_length, len(rep.u.auth_info) - samba.dcerpc.dcerpc.DCERPC_AUTH_TRAILER_LENGTH)
 
         a = self.parse_auth(rep.u.auth_info, auth_context=auth_context)
 
@@ -423,34 +426,34 @@ class RawDCERPCTest(TestCase):
                             pfc_flags=req.pfc_flags |
                             samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_DID_NOT_EXECUTE,
                             auth_length=0)
-            self.assertNotEquals(rep.u.alloc_hint, 0)
-            self.assertEquals(rep.u.context_id, 0)
-            self.assertEquals(rep.u.cancel_count, 0)
-            self.assertEquals(rep.u.flags, 0)
-            self.assertEquals(rep.u.status, alter_fault)
-            self.assertEquals(rep.u.reserved, 0)
-            self.assertEquals(len(rep.u.error_and_verifier), 0)
+            self.assertNotEqual(rep.u.alloc_hint, 0)
+            self.assertEqual(rep.u.context_id, 0)
+            self.assertEqual(rep.u.cancel_count, 0)
+            self.assertEqual(rep.u.flags, 0)
+            self.assertEqual(rep.u.status, alter_fault)
+            self.assertEqual(rep.u.reserved, 0)
+            self.assertEqual(len(rep.u.error_and_verifier), 0)
             return None
         self.verify_pdu(rep, samba.dcerpc.dcerpc.DCERPC_PKT_ALTER_RESP, req.call_id,
                         pfc_flags=req.pfc_flags)
-        self.assertEquals(rep.u.max_xmit_frag, req.u.max_xmit_frag)
-        self.assertEquals(rep.u.max_recv_frag, req.u.max_recv_frag)
-        self.assertEquals(rep.u.assoc_group_id, assoc_group_id)
-        self.assertEquals(rep.u.secondary_address_size, 0)
-        self.assertEquals(rep.u.secondary_address, '')
+        self.assertEqual(rep.u.max_xmit_frag, req.u.max_xmit_frag)
+        self.assertEqual(rep.u.max_recv_frag, req.u.max_recv_frag)
+        self.assertEqual(rep.u.assoc_group_id, assoc_group_id)
+        self.assertEqual(rep.u.secondary_address_size, 0)
+        self.assertEqual(rep.u.secondary_address, '')
         self.assertPadding(rep.u._pad1, 2)
-        self.assertEquals(rep.u.num_results, 1)
-        self.assertEquals(rep.u.ctx_list[0].result,
+        self.assertEqual(rep.u.num_results, 1)
+        self.assertEqual(rep.u.ctx_list[0].result,
                           samba.dcerpc.dcerpc.DCERPC_BIND_ACK_RESULT_ACCEPTANCE)
-        self.assertEquals(rep.u.ctx_list[0].reason,
+        self.assertEqual(rep.u.ctx_list[0].reason,
                           samba.dcerpc.dcerpc.DCERPC_BIND_ACK_REASON_NOT_SPECIFIED)
         self.assertNDRSyntaxEquals(rep.u.ctx_list[0].syntax, ctx.transfer_syntaxes[0])
         if finished:
-            self.assertEquals(rep.auth_length, 0)
+            self.assertEqual(rep.auth_length, 0)
         else:
-            self.assertNotEquals(rep.auth_length, 0)
+            self.assertNotEqual(rep.auth_length, 0)
         self.assertGreaterEqual(len(rep.u.auth_info), samba.dcerpc.dcerpc.DCERPC_AUTH_TRAILER_LENGTH)
-        self.assertEquals(rep.auth_length, len(rep.u.auth_info) - samba.dcerpc.dcerpc.DCERPC_AUTH_TRAILER_LENGTH)
+        self.assertEqual(rep.auth_length, len(rep.u.auth_info) - samba.dcerpc.dcerpc.DCERPC_AUTH_TRAILER_LENGTH)
 
         a = self.parse_auth(rep.u.auth_info, auth_context=auth_context)
 
@@ -523,46 +526,49 @@ class RawDCERPCTest(TestCase):
             if hexdump:
                 sys.stderr.write("stub_in: %d\n%s" % (len(stub_in), self.hexdump(stub_in)))
 
-        pfc_flags = samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_FIRST
-        pfc_flags |= samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_LAST
-        if object is not None:
-            pfc_flags |= samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_OBJECT_UUID
+            pfc_flags = samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_FIRST
+            pfc_flags |= samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_LAST
+            if object is not None:
+                pfc_flags |= samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_OBJECT_UUID
 
-        req = self.generate_request_auth(call_id=call_id,
-                                         context_id=ctx.context_id,
-                                         pfc_flags=pfc_flags,
-                                         object=object,
-                                         opnum=io.opnum(),
-                                         stub=stub_in,
-                                         auth_context=auth_context)
-        if send_req:
+            req = self.generate_request_auth(call_id=call_id,
+                                             context_id=ctx.context_id,
+                                             pfc_flags=pfc_flags,
+                                             object=object,
+                                             opnum=io.opnum(),
+                                             stub=stub_in,
+                                             auth_context=auth_context)
             self.send_pdu(req, ndr_print=ndr_print, hexdump=hexdump)
         if recv_rep:
             (rep, rep_blob) = self.recv_pdu_raw(timeout=timeout,
                                                 ndr_print=ndr_print,
                                                 hexdump=hexdump)
             if fault_status:
-                self.verify_pdu(rep, samba.dcerpc.dcerpc.DCERPC_PKT_FAULT, req.call_id,
+                self.verify_pdu(rep, samba.dcerpc.dcerpc.DCERPC_PKT_FAULT, call_id,
                                 pfc_flags=fault_pfc_flags, auth_length=0)
-                self.assertNotEquals(rep.u.alloc_hint, 0)
-                self.assertEquals(rep.u.context_id, fault_context_id)
-                self.assertEquals(rep.u.cancel_count, 0)
-                self.assertEquals(rep.u.flags, 0)
-                self.assertEquals(rep.u.status, fault_status)
-                self.assertEquals(rep.u.reserved, 0)
-                self.assertEquals(len(rep.u.error_and_verifier), 0)
+                self.assertNotEqual(rep.u.alloc_hint, 0)
+                self.assertEqual(rep.u.context_id, fault_context_id)
+                self.assertEqual(rep.u.cancel_count, 0)
+                self.assertEqual(rep.u.flags, 0)
+                self.assertEqual(rep.u.status, fault_status)
+                self.assertEqual(rep.u.reserved, 0)
+                self.assertEqual(len(rep.u.error_and_verifier), 0)
                 return
 
             expected_auth_length = 0
             if auth_context is not None and \
                auth_context["auth_level"] >= dcerpc.DCERPC_AUTH_LEVEL_PACKET:
-                expected_auth_length = req.auth_length
+                if send_req:
+                    expected_auth_length = req.auth_length
+                else:
+                    expected_auth_length = rep.auth_length
 
-            self.verify_pdu(rep, samba.dcerpc.dcerpc.DCERPC_PKT_RESPONSE, req.call_id,
+
+            self.verify_pdu(rep, samba.dcerpc.dcerpc.DCERPC_PKT_RESPONSE, call_id,
                             auth_length=expected_auth_length)
-            self.assertNotEquals(rep.u.alloc_hint, 0)
-            self.assertEquals(rep.u.context_id, req.u.context_id & 0xff)
-            self.assertEquals(rep.u.cancel_count, 0)
+            self.assertNotEqual(rep.u.alloc_hint, 0)
+            self.assertEqual(rep.u.context_id, ctx.context_id & 0xff)
+            self.assertEqual(rep.u.cancel_count, 0)
             self.assertGreaterEqual(len(rep.u.stub_and_verifier), rep.u.alloc_hint)
             stub_out = self.check_response_auth(rep, rep_blob, auth_context)
             self.assertEqual(len(stub_out), rep.u.alloc_hint)
@@ -783,10 +789,10 @@ class RawDCERPCTest(TestCase):
             sys.stderr.write("parse_auth: %s" % samba.ndr.ndr_print(a))
 
         if auth_context is not None:
-            self.assertEquals(a.auth_type, auth_context["auth_type"])
-            self.assertEquals(a.auth_level, auth_context["auth_level"])
-            self.assertEquals(a.auth_reserved, 0)
-            self.assertEquals(a.auth_context_id, auth_context["auth_context_id"])
+            self.assertEqual(a.auth_type, auth_context["auth_type"])
+            self.assertEqual(a.auth_level, auth_context["auth_level"])
+            self.assertEqual(a.auth_reserved, 0)
+            self.assertEqual(a.auth_context_id, auth_context["auth_context_id"])
 
             self.assertLessEqual(a.auth_pad_length, dcerpc.DCERPC_AUTH_PAD_ALIGNMENT)
             self.assertLessEqual(a.auth_pad_length, stub_len)
@@ -797,11 +803,11 @@ class RawDCERPCTest(TestCase):
                             auth_pad_length=None):
 
         if auth_context is None:
-            self.assertEquals(rep.auth_length, 0)
+            self.assertEqual(rep.auth_length, 0)
             return rep.u.stub_and_verifier
 
         if auth_context["auth_level"] == dcerpc.DCERPC_AUTH_LEVEL_CONNECT:
-            self.assertEquals(rep.auth_length, 0)
+            self.assertEqual(rep.auth_length, 0)
             return rep.u.stub_and_verifier
 
         self.assertGreater(rep.auth_length, 0)
@@ -818,8 +824,8 @@ class RawDCERPCTest(TestCase):
                                         auth_context=auth_context,
                                         stub_len=len(rep_data))
         if auth_pad_length is not None:
-            self.assertEquals(rep_auth_info.auth_pad_length, auth_pad_length)
-        self.assertEquals(rep_auth_info.credentials, rep_sig)
+            self.assertEqual(rep_auth_info.auth_pad_length, auth_pad_length)
+        self.assertEqual(rep_auth_info.credentials, rep_sig)
 
         if auth_context["auth_level"] >= dcerpc.DCERPC_AUTH_LEVEL_PRIVACY:
             # TODO: not yet supported here
@@ -836,9 +842,11 @@ class RawDCERPCTest(TestCase):
                      rpc_vers_minor=0,
                      pfc_flags=(samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_FIRST |
                                 samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_LAST),
-                     drep=[samba.dcerpc.dcerpc.DCERPC_DREP_LE, 0, 0, 0],
+                     drep=None,
                      ndr_print=None, hexdump=None):
 
+        if drep is None:
+            drep = [samba.dcerpc.dcerpc.DCERPC_DREP_LE, 0, 0, 0]
         if getattr(payload, 'auth_info', None):
             ai = payload.auth_info
         else:
@@ -919,19 +927,19 @@ class RawDCERPCTest(TestCase):
         req_data = req_blob[ofs_stub:ofs_trailer]
         req_whole = req_blob[0:ofs_sig]
 
-        if auth_context["auth_level"] >= dcerpc.DCERPC_AUTH_LEVEL_PRIVACY:
+        if auth_context["g_auth_level"] >= dcerpc.DCERPC_AUTH_LEVEL_PRIVACY:
             # TODO: not yet supported here
             self.assertTrue(False)
-        elif auth_context["auth_level"] >= dcerpc.DCERPC_AUTH_LEVEL_PACKET:
+        elif auth_context["g_auth_level"] >= dcerpc.DCERPC_AUTH_LEVEL_PACKET:
             req_sig = auth_context["gensec"].sign_packet(req_data, req_whole)
-        elif auth_context["auth_level"] >= dcerpc.DCERPC_AUTH_LEVEL_CONNECT:
-            self.assertEquals(auth_context["auth_type"],
+        elif auth_context["g_auth_level"] >= dcerpc.DCERPC_AUTH_LEVEL_CONNECT:
+            self.assertEqual(auth_context["auth_type"],
                               dcerpc.DCERPC_AUTH_TYPE_NTLMSSP)
             req_sig = b"\x01" +b"\x00" *15
         else:
             return req
-        self.assertEquals(len(req_sig), req.auth_length)
-        self.assertEquals(len(req_sig), sig_size)
+        self.assertEqual(len(req_sig), req.auth_length)
+        self.assertEqual(len(req_sig), sig_size)
 
         stub_sig_ofs = len(req.u.stub_and_verifier) - sig_size
         stub = req.u.stub_and_verifier[0:stub_sig_ofs] + req_sig
@@ -944,9 +952,11 @@ class RawDCERPCTest(TestCase):
                    rpc_vers_minor=0,
                    pfc_flags=(samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_FIRST |
                               samba.dcerpc.dcerpc.DCERPC_PFC_FLAG_LAST),
-                   drep=[samba.dcerpc.dcerpc.DCERPC_DREP_LE, 0, 0, 0],
+                   drep=None,
                    auth_length=None):
 
+        if drep is None:
+            drep = [samba.dcerpc.dcerpc.DCERPC_DREP_LE, 0, 0, 0]
         self.assertIsNotNone(p, "No valid pdu")
 
         if getattr(p.u, 'auth_info', None):
@@ -978,10 +988,12 @@ class RawDCERPCTest(TestCase):
                       max_xmit_frag=None,
                       max_recv_frag=None,
                       assoc_group_id=0,
-                      ctx_list=[],
+                      ctx_list=None,
                       auth_info=b"",
                       ndr_print=None, hexdump=None):
 
+        if ctx_list is None:
+            ctx_list = []
         if max_xmit_frag is None:
             max_xmit_frag=self.max_xmit_frag
         if max_recv_frag is None:
@@ -1009,10 +1021,12 @@ class RawDCERPCTest(TestCase):
                        max_xmit_frag=None,
                        max_recv_frag=None,
                        assoc_group_id=0,
-                       ctx_list=[],
+                       ctx_list=None,
                        auth_info=b"",
                        ndr_print=None, hexdump=None):
 
+        if ctx_list is None:
+            ctx_list = []
         if max_xmit_frag is None:
             max_xmit_frag=self.max_xmit_frag
         if max_recv_frag is None:
@@ -1147,7 +1161,7 @@ class RawDCERPCTest(TestCase):
         return
 
     def assertPadding(self, pad, length):
-        self.assertEquals(len(pad), length)
+        self.assertEqual(len(pad), length)
         #
         # sometimes windows sends random bytes
         #
@@ -1157,7 +1171,7 @@ class RawDCERPCTest(TestCase):
         if self.ignore_random_pad:
             return
         zero_pad = b'\0' * length
-        self.assertEquals(pad, zero_pad)
+        self.assertEqual(pad, zero_pad)
 
     def assertEqualsStrLower(self, s1, s2):
-        self.assertEquals(str(s1).lower(), str(s2).lower())
+        self.assertEqual(str(s1).lower(), str(s2).lower())
